import { Injectable, EventEmitter, Inject, Pipe, ChangeDetectorRef, Component, NgModule, CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import { HttpClient, HttpHeaders, HttpClientModule } from '@angular/common/http';
import { TranslateService, TranslatePipe, TranslateModule } from '@ngx-translate/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { BrowserModule } from '@angular/platform-browser';
import { ModalModule } from 'ngx-bootstrap/modal';
import { PaginationModule } from 'ngx-bootstrap/pagination';
import { NgxPaginationModule } from 'ngx-pagination';
import { BsDatepickerModule } from 'ngx-bootstrap/datepicker';
import { AccordionModule } from 'ngx-bootstrap/accordion';
import { TooltipModule } from 'ngx-bootstrap/tooltip';
import { TabsModule } from 'ngx-bootstrap/tabs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class StorageService {
    constructor() {
        this._storage = {};
        this._inProgress = {};
    }
    /**
     * @param {?} key
     * @return {?}
     */
    get(key) {
        return this._storage[key];
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    set(key, value) {
        this._storage[key] = value;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    remove(key) {
        delete this._storage[key];
    }
    /**
     * @param {?} match
     * @return {?}
     */
    removeMatched(match) {
        Object.keys(this._storage).forEach((key) => {
            if (key.indexOf(match) !== -1) {
                this.remove(key);
            }
        });
        Object.keys(this._inProgress).forEach((key) => {
            if (key.indexOf(match) !== -1) {
                this.deleteProgress(key);
            }
        });
    }
    /**
     * @return {?}
     */
    clearCache() {
        this._inProgress = {};
        this._storage = {};
    }
    /**
     * @param {?} key
     * @return {?}
     */
    getProgress(key) {
        return this._inProgress[key];
    }
    /**
     * @param {?} key
     * @return {?}
     */
    deleteProgress(key) {
        delete this._inProgress[key];
    }
    /**
     * @param {?} key
     * @return {?}
     */
    setProgress(key) {
        this._inProgress[key] = new EventEmitter();
    }
}
StorageService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
StorageService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class FVEnviroment {
    /**
     * @param {?=} debug
     * @param {?=} apiUrl
     * @param {?=} defaultLanguage
     * @param {?=} languages
     * @param {?=} lowerCaseTranslates
     */
    constructor(debug, apiUrl, defaultLanguage, languages, lowerCaseTranslates) {
        this.debug = debug;
        this.apiUrl = apiUrl;
        this.defaultLanguage = defaultLanguage;
        this.languages = languages;
        this.lowerCaseTranslates = lowerCaseTranslates;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ApiResponse {
    /**
     * @param {?=} data
     * @param {?=} error
     */
    constructor(data, error) {
        this.data = data;
        this.error = error;
    }
}
class ApiService extends StorageService {
    /**
     * @param {?} httpClient
     * @param {?} enviroment
     */
    constructor(httpClient, enviroment) {
        super();
        this.httpClient = httpClient;
        this.enviroment = enviroment;
    }
    /**
     * Make GET request to api
     * @param {?} point
     * @param {?=} args
     * @param {?=} parameters
     * @param {?=} options
     * @return {?}
     */
    get(point, args = [], parameters = {}, options = {}) {
        return this.request('GET', point, parameters, options, undefined, ...args);
    }
    /**
     * Make DELETE request to api
     * @param {?} point
     * @param {?=} args
     * @param {?=} parameters
     * @param {?=} options
     * @return {?}
     */
    delete(point, args = [], parameters = {}, options = {}) {
        return this.request('DELETE', point, parameters, options, undefined, ...args);
    }
    /**
     * Make OPTIONS request to api
     * @param {?} point
     * @param {?=} args
     * @param {?=} parameters
     * @param {?=} options
     * @return {?}
     */
    options(point, args = [], parameters = {}, options = {}) {
        return this.request('OPTIONS', point, parameters, options, undefined, ...args);
    }
    /**
     * Make POST request to api
     * @param {?} point
     * @param {?=} args
     * @param {?=} parameters
     * @param {?=} body
     * @param {?=} options
     * @return {?}
     */
    post(point, args = [], parameters = {}, body = {}, options = {}) {
        return this.request('POST', point, parameters, options, body, ...args);
    }
    /**
     * Make PATCH request to api
     * @param {?} point
     * @param {?=} args
     * @param {?=} parameters
     * @param {?=} body
     * @param {?=} options
     * @return {?}
     */
    patch(point, args = [], parameters = {}, body = {}, options = {}) {
        return this.request('PATCH', point, parameters, options, body, ...args);
    }
    /**
     * Make PUT request to api
     * @param {?} point
     * @param {?=} args
     * @param {?=} parameters
     * @param {?=} body
     * @param {?=} options
     * @return {?}
     */
    put(point, args = [], parameters = {}, body = {}, options = {}) {
        return this.request('PUT', point, parameters, options, body, ...args);
    }
    /**
     * Return generated url
     * @param {?} point
     * @param {?=} args
     * @param {?=} parameters
     * @param {?=} body
     * @param {?=} options
     * @return {?}
     */
    getURL(point, args = [], parameters = {}, body = {}, options = {}) {
        return this.request('getURL', point, parameters, options, body, ...args);
    }
    /**
     * @return {?}
     */
    get apiRenameList() {
        return ['Name', 'Title'];
    }
    /**
     * @return {?}
     */
    get API() {
        return this.enviroment.apiUrl;
    }
    /**
     * @param {?} origin
     * @param {?} target
     * @return {?}
     */
    rename(origin, target) {
        this.languages.forEach((lang) => {
            const /** @type {?} */ from = `${origin}_${lang}`;
            const /** @type {?} */ to = `${origin}`;
            if (target.hasOwnProperty(from) && !target.hasOwnProperty(to)) {
                target[to] = target[from];
                delete target[from];
            }
        });
    }
    /**
     * Transform subscription to promise
     * For using same behavior throught whole application
     * @param {?} subscription
     * @return {?}
     */
    subscribeToPromise(subscription) {
        return new Promise((resolve, reject) => {
            subscription.subscribe(resolve, reject);
        });
    }
    /**
     * Generates final url from api_url + view + query parameters;
     * @param {?} url
     * @param {?} parameters
     * @param {...?} args
     * @return {?}
     */
    generateUrl(url, parameters, ...args) {
        const /** @type {?} */ endUrl = `${url}${args.length ? `/${args.join('/')}` : ''}`;
        const /** @type {?} */ queryParams = Object.keys(parameters).map((key) => `${key}=${this.sanitize(parameters[key])}`).join('&');
        return `${endUrl}${queryParams ? `?${queryParams}` : ''}`;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    sanitize(value) {
        const /** @type {?} */ exists = value !== undefined && value !== null;
        const /** @type {?} */ isNum = (val) => val.constructor === Number;
        const /** @type {?} */ isString = (val) => val.constructor === String;
        if (!exists) {
            return value;
        }
        if (isString(value)) {
            return value.trim();
        }
        if (isNum(value)) {
            return value;
        }
        return JSON.stringify(value);
    }
    /**
     * If data contains fields like Name_ru/Name_ro/Name_en transform it into Name.
     * @param {?} response
     * @return {?}
     */
    resolve(response) {
        if (response && response.data && Array.isArray(response.data)) {
            response.data.forEach((item) => {
                this.apiRenameList.forEach((renameOrigin) => {
                    this.rename(renameOrigin, item);
                });
            });
        }
        return {
            data: response,
            error: null,
        };
    }
    /**
     * Private Abstract request function
     * @param {?} method
     * @param {?} point
     * @param {?} parameters
     * @param {?} options
     * @param {?=} body
     * @param {...?} args
     * @return {?}
     */
    request(method, point, parameters, options, body = null, ...args) {
        method = method.toLowerCase();
        const /** @type {?} */ url = this.generateUrl(this.API + point, parameters, ...args);
        if (method === 'geturl') {
            return url;
        }
        return new Promise((resolve, reject) => {
            const /** @type {?} */ cache = super.get(url);
            const /** @type {?} */ progress = super.getProgress(url);
            const /** @type {?} */ skipCache = options['skipCache'] || false;
            if (cache && skipCache !== true) {
                // Extract response from already finished request
                return resolve(cache);
            }
            else if (progress && skipCache !== true) {
                // Extract response from already existing request in progress
                progress.subscribe((response) => {
                    super.deleteProgress(url);
                    const /** @type {?} */ resolvedData = this.resolve(response);
                    return resolve(resolvedData);
                });
            }
            else {
                let /** @type {?} */ callParameters = [];
                // Add headers
                const /** @type {?} */ auth = JSON.parse(localStorage.getItem('auth'));
                const /** @type {?} */ headers = options['headers'] || {};
                if (auth) {
                    headers['Authorization'] = `${auth.tokenType} ${auth.token}`;
                }
                const /** @type {?} */ userContext = JSON.parse(localStorage.getItem('userContext'));
                if (!options['skipRole'] && userContext) {
                    headers['x-Current-Role'] = `${userContext.role}`;
                }
                options['headers'] = new HttpHeaders(headers);
                if (method === 'get' || method === 'delete' || method === 'options') {
                    callParameters = [url, options];
                    // Request not in cache/progress so we set it.
                    super.setProgress(url);
                }
                else if (method === 'patch' || method === 'put' || method === 'post') {
                    callParameters = [url, body || {}, options];
                }
                this.subscribeToPromise(this.httpClient[method](...callParameters)).then((rawData) => {
                    const /** @type {?} */ resolvedData = this.resolve(rawData);
                    if (method === 'get' || method === 'delete' || method === 'options') {
                        super.set(url, resolvedData);
                        const /** @type {?} */ prog = super.getProgress(url);
                        if (prog) {
                            prog.emit(resolvedData);
                            super.deleteProgress(url);
                        }
                    }
                    return resolve(resolvedData);
                }).catch((e) => {
                    resolve({ data: null, error: e });
                });
            }
        });
    }
    /**
     * @return {?}
     */
    get languages() {
        return ['en', 'ro', 'ru'];
    }
}
ApiService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ApiService.ctorParameters = () => [
    { type: HttpClient, },
    { type: FVEnviroment, decorators: [{ type: Inject, args: ['enviroment',] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Breadcrumbps {
    /**
     * @param {?=} current
     * @param {?=} path
     */
    constructor(current, path) {
        this.current = current;
        this.path = path;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ComponentConfig {
    /**
     * @param {?=} controls
     * @param {?=} docketItems
     * @param {?=} errorMessage
     * @param {?=} height
     * @param {?=} isError
     * @param {?=} items
     * @param {?=} model
     * @param {?=} readonly
     * @param {?=} recordData
     * @param {?=} title
     * @param {?=} width
     * @param {?=} windowMask
     */
    constructor(controls, docketItems, errorMessage, height, isError, items, model, readonly, recordData, title, width, windowMask) {
        this.controls = controls;
        this.docketItems = docketItems;
        this.errorMessage = errorMessage;
        this.height = height;
        this.isError = isError;
        this.items = items;
        this.model = model;
        this.readonly = readonly;
        this.recordData = recordData;
        this.title = title;
        this.width = width;
        this.windowMask = windowMask;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

/**
 * @record
 */

class Control {
    /**
     * @param {?=} bindingSet
     * @param {?=} handler
     * @param {?=} isHidden
     * @param {?=} isReadonly
     * @param {?=} isDisabled
     * @param {?=} isRequired
     * @param {?=} key
     * @param {?=} onCreateValue
     * @param {?=} onUpdateValue
     * @param {?=} referenceName
     * @param {?=} selectAction
     * @param {?=} view
     * @param {?=} controlType
     * @param {?=} dataSourceType
     * @param {?=} label
     * @param {?=} options
     * @param {?=} showAs
     * @param {?=} columns
     * @param {?=} сontrolType
     * @param {?=} editableGrid
     * @param {?=} controlName
     * @param {?=} description
     * @param {?=} eventJs
     * @param {?=} content
     * @param {?=} watermark
     * @param {?=} controls
     * @param {?=} placeholder
     * @param {?=} size
     * @param {?=} inline
     * @param {?=} isMultiple
     */
    constructor(bindingSet, handler, isHidden, isReadonly, isDisabled, isRequired, key, onCreateValue, onUpdateValue, referenceName, selectAction, view, controlType, dataSourceType, label, options, showAs, columns, сontrolType, editableGrid, controlName, description, eventJs, content, watermark, controls, placeholder, size, inline, isMultiple) {
        this.bindingSet = bindingSet;
        this.handler = handler;
        this.isHidden = isHidden;
        this.isReadonly = isReadonly;
        this.isDisabled = isDisabled;
        this.isRequired = isRequired;
        this.key = key;
        this.onCreateValue = onCreateValue;
        this.onUpdateValue = onUpdateValue;
        this.referenceName = referenceName;
        this.selectAction = selectAction;
        this.view = view;
        this.controlType = controlType;
        this.dataSourceType = dataSourceType;
        this.label = label;
        this.options = options;
        this.showAs = showAs;
        this.columns = columns;
        this.сontrolType = сontrolType;
        this.editableGrid = editableGrid;
        this.controlName = controlName;
        this.description = description;
        this.eventJs = eventJs;
        this.content = content;
        this.watermark = watermark;
        this.controls = controls;
        this.placeholder = placeholder;
        this.size = size;
        this.inline = inline;
        this.isMultiple = isMultiple;
    }
}
class ControlType {
    constructor() { }
    /**
     * @param {?} num
     * @return {?}
     */
    static controlType(num) {
        const /** @type {?} */ enums = ControlType.EFormElement;
        return Object.keys(enums).find((key) => enums[key] === num);
    }
    /**
     * @param {?} num
     * @return {?}
     */
    static controlGroupType(num) {
        return ControlType.isInputField(num) && 'input'
            || ControlType.isFormField(num) && 'form'
            || ControlType.isDesignField(num) && 'design'
            || 'undefined';
    }
    /**
     * @param {?} num
     * @return {?}
     */
    static isInputField(num) {
        const /** @type {?} */ controlType = ControlType.controlType(num);
        const /** @type {?} */ inputFields = ControlType.inputFields;
        return inputFields.indexOf(controlType) !== -1;
    }
    /**
     * @param {?} num
     * @return {?}
     */
    static isFormField(num) {
        const /** @type {?} */ controlType = ControlType.controlType(num);
        const /** @type {?} */ formFields = ControlType.formFields;
        return formFields.indexOf(controlType) !== -1;
    }
    /**
     * @param {?} num
     * @return {?}
     */
    static isDesignField(num) {
        const /** @type {?} */ controlType = ControlType.controlType(num);
        const /** @type {?} */ designFields = ControlType.designFields;
        return designFields.indexOf(controlType) !== -1;
    }
    /**
     * @param {?} num
     * @return {?}
     */
    static isDateType(num) {
        const /** @type {?} */ dateInputs = [9, 10, 11];
        return dateInputs.indexOf(num) !== -1;
    }
}
ControlType.inputFields = [
    'TextField',
    'TextBox',
    'TextArea',
    'EmailField',
    'NumberField',
    'MoneyField',
    'PositiveNumber',
    'CheckBox',
    'Toggle',
    'RadioButton',
    'UploadFile',
    'ListField',
    'PickList',
    'Lookup',
    'RadioButtonList',
    'DropDownList',
    'Date',
    'Time',
    'DateTime',
];
ControlType.formFields = [
    'Label',
    'Button',
    'EditableGrid',
    'MenuGrid',
    'ProgressBar',
];
ControlType.designFields = [
    'HiddenField',
    'TableLayout',
    'BootstrapRow',
    'SectionLayout',
    'FlexRow',
    'HtmlContent',
    'FormTab',
    'Form',
    'CustomControl',
];
ControlType.EFormElement = Object.freeze({
    UNDEFINED: 0,
    TextBox: 1,
    TextArea: 2,
    Button: 3,
    CheckBox: 4,
    UploadFile: 5,
    RadioButton: 6,
    RadioButtonList: 7,
    DropDownList: 8,
    Date: 9,
    Time: 10,
    DateTime: 11,
    TableLayout: 13,
    BootstrapRow: 14,
    FlexRow: 15,
    Toggle: 16,
    HiddenField: 50,
    Label: 100,
    TextField: 101,
    ListField: 107,
    PickList: 108,
    Lookup: 109,
    EditableGrid: 114,
    EmailField: 121,
    NumberField: 122,
    MoneyField: 123,
    PositiveNumber: 124,
    HtmlContent: 200,
    CustomControl: 201,
    SectionLayout: 300,
    MenuGrid: 400,
    ProgressBar: 401,
    FormTab: 1000,
    Form: 1001,
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Dock {
    /**
     * @param {?=} dock
     * @param {?=} items
     */
    constructor(dock, items) {
        this.dock = dock;
        this.items = items;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class DockItem {
    /**
     * @param {?=} dialogData
     * @param {?=} disabled
     * @param {?=} iconCls
     * @param {?=} isDblClick
     * @param {?=} listeners
     * @param {?=} reference
     * @param {?=} referenceName
     * @param {?=} selectionEnable
     * @param {?=} text
     * @param {?=} tooltip
     * @param {?=} btnClass
     */
    constructor(dialogData, disabled, iconCls, isDblClick, listeners, reference, referenceName, selectionEnable, text, tooltip, btnClass) {
        this.dialogData = dialogData;
        this.disabled = disabled;
        this.iconCls = iconCls;
        this.isDblClick = isDblClick;
        this.listeners = listeners;
        this.reference = reference;
        this.referenceName = referenceName;
        this.selectionEnable = selectionEnable;
        this.text = text;
        this.tooltip = tooltip;
        this.btnClass = btnClass;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Extract {
    /**
     * @param {?=} table
     * @param {?=} view
     */
    constructor(table, view) {
        this.table = table;
        this.view = view;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Field {
    /**
     * @param {?=} autoLoad
     * @param {?=} bindingSet
     * @param {?=} controlType
     * @param {?=} dbType
     * @param {?=} description
     * @param {?=} editableGrid
     * @param {?=} extract
     * @param {?=} format
     * @param {?=} maxValue
     * @param {?=} minValue
     * @param {?=} name
     * @param {?=} options
     * @param {?=} readOnly
     * @param {?=} required
     * @param {?=} size
     * @param {?=} title
     * @param {?=} watermark
     * @param {?=} value
     * @param {?=} label
     */
    constructor(autoLoad, bindingSet, controlType, dbType, description, editableGrid, extract, format, maxValue, minValue, name, options, readOnly, required, size, title, watermark, value, label) {
        this.autoLoad = autoLoad;
        this.bindingSet = bindingSet;
        this.controlType = controlType;
        this.dbType = dbType;
        this.description = description;
        this.editableGrid = editableGrid;
        this.extract = extract;
        this.format = format;
        this.maxValue = maxValue;
        this.minValue = minValue;
        this.name = name;
        this.options = options;
        this.readOnly = readOnly;
        this.required = required;
        this.size = size;
        this.title = title;
        this.watermark = watermark;
        this.value = value;
        this.label = label;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Item {
    /**
     * @param {?=} tabs
     * @param {?=} handlers
     */
    constructor(tabs, handlers) {
        this.tabs = tabs;
        this.handlers = handlers;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Model {
    /**
     * @param {?=} autoLoad
     * @param {?=} entityName
     * @param {?=} idProperty
     * @param {?=} fields
     * @param {?=} proxy
     */
    constructor(autoLoad, entityName, idProperty, fields, proxy) {
        this.autoLoad = autoLoad;
        this.entityName = entityName;
        this.idProperty = idProperty;
        this.fields = fields;
        this.proxy = proxy;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Path {
    /**
     * @param {?=} key
     * @param {?=} value
     */
    constructor(key, value) {
        this.key = key;
        this.value = value;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Proxy {
    /**
     * @param {?=} url
     */
    constructor(url) {
        this.url = url;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Tab {
    /**
     * @param {?=} controls
     * @param {?=} key
     * @param {?=} label
     * @param {?=} сontrolType
     */
    constructor(controls, key, label, сontrolType) {
        this.controls = controls;
        this.key = key;
        this.label = label;
        this.сontrolType = сontrolType;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Task {
    /**
     * @param {?=} breadcrumbps
     * @param {?=} clientComponent
     * @param {?=} componentConfig
     * @param {?=} dockedItems
     * @param {?=} processInstanceId
     * @param {?=} processName
     * @param {?=} stateId
     * @param {?=} status
     * @param {?=} taskId
     * @param {?=} componentClassName
     */
    constructor(breadcrumbps, clientComponent, componentConfig, dockedItems, processInstanceId, processName, stateId, status, taskId, componentClassName) {
        this.breadcrumbps = breadcrumbps;
        this.clientComponent = clientComponent;
        this.componentConfig = componentConfig;
        this.dockedItems = dockedItems;
        this.processInstanceId = processInstanceId;
        this.processName = processName;
        this.stateId = stateId;
        this.status = status;
        this.taskId = taskId;
        this.componentClassName = componentClassName;
    }
}
Task.TASK_DONE_STATUS = 1;
class FVDoneResponse {
    /**
     * @param {?=} done
     * @param {?=} close
     * @param {?=} data
     * @param {?=} action
     * @param {?=} group
     */
    constructor(done, close, data, action, group) {
        this.done = done;
        this.close = close;
        this.data = data;
        this.action = action;
        this.group = group;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

/**
 * @record
 */

class GridColumn {
    /**
     * @param {?=} align
     * @param {?=} dataIndex
     * @param {?=} flex
     * @param {?=} hidden
     * @param {?=} text
     * @param {?=} type
     * @param {?=} width
     */
    constructor(align, dataIndex, flex, hidden, text, type, width) {
        this.align = align;
        this.dataIndex = dataIndex;
        this.flex = flex;
        this.hidden = hidden;
        this.text = text;
        this.type = type;
        this.width = width;
    }
}
class Store {
    /**
     * @param {?=} autoLoad
     * @param {?=} formFilter
     * @param {?=} passColumns
     * @param {?=} proxy
     * @param {?=} remoteFilter
     * @param {?=} remoteSort
     * @param {?=} title
     */
    constructor(autoLoad, formFilter, passColumns, proxy, remoteFilter, remoteSort, title) {
        this.autoLoad = autoLoad;
        this.formFilter = formFilter;
        this.passColumns = passColumns;
        this.proxy = proxy;
        this.remoteFilter = remoteFilter;
        this.remoteSort = remoteSort;
        this.title = title;
    }
}
class GridComponentConfig {
    /**
     * @param {?=} columns
     * @param {?=} store
     * @param {?=} dockedItems
     * @param {?=} idFilterProperty
     * @param {?=} model
     * @param {?=} predefinedFilter
     * @param {?=} title
     * @param {?=} editHandlers
     */
    constructor(columns, store, dockedItems, idFilterProperty, model, predefinedFilter, title, editHandlers) {
        this.columns = columns;
        this.store = store;
        this.dockedItems = dockedItems;
        this.idFilterProperty = idFilterProperty;
        this.model = model;
        this.predefinedFilter = predefinedFilter;
        this.title = title;
        this.editHandlers = editHandlers;
    }
}
class EditHandlers {
    /**
     * @param {?=} autoNewRow
     * @param {?=} allowInlineEdit
     * @param {?=} addAction
     * @param {?=} deleteAction
     * @param {?=} editAction
     * @param {?=} selectActionRo
     */
    constructor(autoNewRow, allowInlineEdit, addAction, deleteAction, editAction, selectActionRo) {
        this.autoNewRow = autoNewRow;
        this.allowInlineEdit = allowInlineEdit;
        this.addAction = addAction;
        this.deleteAction = deleteAction;
        this.editAction = editAction;
        this.selectActionRo = selectActionRo;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class GridConfig {
    /**
     * @param {?=} componentClassName
     * @param {?=} componentConfig
     * @param {?=} id
     * @param {?=} filters
     * @param {?=} parent
     * @param {?=} model
     * @param {?=} editable
     * @param {?=} hideFilter
     * @param {?=} notClickable
     * @param {?=} addAction
     * @param {?=} editAction
     * @param {?=} deleteAction
     */
    constructor(componentClassName, componentConfig, id, filters, parent, model, editable, hideFilter, notClickable, addAction, editAction, deleteAction) {
        this.componentClassName = componentClassName;
        this.componentConfig = componentConfig;
        this.id = id;
        this.filters = filters;
        this.parent = parent;
        this.model = model;
        this.editable = editable;
        this.hideFilter = hideFilter;
        this.notClickable = notClickable;
        this.addAction = addAction;
        this.editAction = editAction;
        this.deleteAction = deleteAction;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Lang {
    /**
     * @param {?} Name
     * @param {?} Code
     * @param {?=} Icon
     */
    constructor(Name, Code, Icon = 'fa fa-globe') {
        this.Name = Name;
        this.Code = Code;
        this.Icon = Icon;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Menu {
    /**
     * @param {?=} id
     * @param {?=} defaultNode
     * @param {?=} outlookView
     * @param {?=} title
     * @param {?=} windowMask
     * @param {?=} windowMaskTable
     * @param {?=} level
     * @param {?=} icon
     * @param {?=} folders
     */
    constructor(id, defaultNode, outlookView, title, windowMask, windowMaskTable, level, icon, folders) {
        this.id = id;
        this.defaultNode = defaultNode;
        this.outlookView = outlookView;
        this.title = title;
        this.windowMask = windowMask;
        this.windowMaskTable = windowMaskTable;
        this.level = level;
        this.icon = icon;
        this.folders = folders;
    }
}
class Folder {
    /**
     * @param {?=} type
     * @param {?=} name
     * @param {?=} title
     * @param {?=} icon
     * @param {?=} showRecordsCount
     * @param {?=} renderRootFolder
     * @param {?=} dynamicTreeOptions
     * @param {?=} count
     * @param {?=} window
     * @param {?=} folders
     * @param {?=} model
     */
    constructor(type, name, title, icon, showRecordsCount, renderRootFolder, dynamicTreeOptions, count, window, folders, model) {
        this.type = type;
        this.name = name;
        this.title = title;
        this.icon = icon;
        this.showRecordsCount = showRecordsCount;
        this.renderRootFolder = renderRootFolder;
        this.dynamicTreeOptions = dynamicTreeOptions;
        this.count = count;
        this.window = window;
        this.folders = folders;
        this.model = model;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class LanguageService {
    /**
     * @param {?} enviroment
     * @param {?} translate
     * @param {?} apiService
     */
    constructor(enviroment, translate, apiService) {
        this.enviroment = enviroment;
        this.translate = translate;
        this.apiService = apiService;
        this.languageChanged = new EventEmitter();
        this._firstTime = true;
        this._language = this.loadSavedLanguage() || this.defaultLanguage;
        this._translations = {};
        this.defaultLanguage = enviroment.defaultLanguage || new Lang('English', 'en', 'fa fa-flag');
        this._languages = this.enviroment.languages || [];
        this.translate.setDefaultLang(this.defaultLanguage.Code);
    }
    /**
     * @return {?}
     */
    loadSavedLanguage() {
        return JSON.parse(localStorage.getItem('language')) || false;
    }
    /**
     * @param {?=} key
     * @return {?}
     */
    getTranslate(key = '') {
        return new Promise((resolve, reject) => {
            this.translate.get(key.toLowerCase()).subscribe(resolve, reject);
        });
    }
    /**
     * @param {?} langCode
     * @param {?} transation
     * @return {?}
     */
    addTranslate(langCode, transation) {
        this._translations[langCode] = transation;
    }
    /**
     * @return {?}
     */
    get languages() {
        return Promise.resolve(this._languages);
    }
    /**
     * @return {?}
     */
    get language() {
        return this._language;
    }
    /**
     * @param {?} lang
     * @return {?}
     */
    set language(lang) {
        this.setupLanguage(lang);
    }
    /**
     * @param {?} lang
     * @return {?}
     */
    setupLanguage(lang) {
        if (!this.language || !this.language.Code) {
            this._language = lang;
        }
        this.translate.use(this.language.Code);
        this.translate.addLangs([this.language.Code]);
        const /** @type {?} */ translation = this._translations[lang.Code];
        const /** @type {?} */ handleTranslation = (_translation) => {
            this.translate.setTranslation(this.language.Code, _translation);
            localStorage.setItem('language', JSON.stringify(lang));
            if (lang.Code !== this._language.Code) {
                this._language = lang;
                this.languageChanged.emit(lang);
            }
        };
        if (translation) {
            handleTranslation(translation);
            return;
        }
        this.apiService.get('translate', [this.language.Code]).then((translations) => {
            handleTranslation(translations);
        });
    }
}
LanguageService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
LanguageService.ctorParameters = () => [
    { type: FVEnviroment, decorators: [{ type: Inject, args: ['enviroment',] },] },
    { type: TranslateService, },
    { type: ApiService, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

class FVNotificationService {
    constructor() {
        this.notifyEvent = new EventEmitter();
    }
    /**
     * @param {?} message
     * @param {?=} title
     * @return {?}
     */
    error(message, title) {
        this.emit({ type: 'error', message, title });
    }
    /**
     * @param {?} message
     * @param {?=} title
     * @return {?}
     */
    info(message, title) {
        this.emit({ type: 'info', message, title });
    }
    /**
     * @param {?} message
     * @param {?=} title
     * @return {?}
     */
    success(message, title) {
        this.emit({ type: 'success', message, title });
    }
    /**
     * @param {?} message
     * @param {?=} title
     * @return {?}
     */
    warning(message, title) {
        this.emit({ type: 'warning', message, title });
    }
    /**
     * @param {?} cb
     * @return {?}
     */
    subscription(cb) {
        return this.notifyEvent.subscribe(cb);
    }
    /**
     * @param {?} data
     * @return {?}
     */
    emit(data) {
        this.notifyEvent.emit(data);
    }
}
FVNotificationService.decorators = [
    { type: Injectable },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class LowerTranslatePipe {
    /**
     * @param {?} translate
     * @param {?} _ref
     */
    constructor(translate, _ref) {
        this.translate = translate;
        this._ref = _ref;
        this.translatePipe = new TranslatePipe(this.translate, this._ref);
    }
    /**
     * @param {?=} value
     * @param {...?} args
     * @return {?}
     */
    transform(value = '', ...args) {
        const /** @type {?} */ isString = typeof value === 'string';
        let /** @type {?} */ newValue = value;
        if (isString) {
            newValue = value.toLowerCase() || ' ';
        }
        const /** @type {?} */ translation = this.translatePipe.transform(newValue, ...args);
        if (translation === newValue && isString) {
            return value;
        }
        return translation;
    }
}
LowerTranslatePipe.decorators = [
    { type: Pipe, args: [{ name: 'ltranslate', pure: false },] },
];
/** @nocollapse */
LowerTranslatePipe.ctorParameters = () => [
    { type: TranslateService, },
    { type: ChangeDetectorRef, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class FBRendererComponent {
    constructor() { }
}
FBRendererComponent.decorators = [
    { type: Component, args: [{
                selector: 'fv-renderer',
                template: `form builder`,
                styles: [``],
            },] },
];
/** @nocollapse */
FBRendererComponent.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class FormBuilderModule {
    /**
     * @param {?} enviroment
     * @return {?}
     */
    static forRoot(enviroment) {
        return {
            ngModule: FormBuilderModule,
            providers: [
                ApiService,
                StorageService,
                LanguageService,
                FVNotificationService,
                { provide: 'enviroment', useValue: enviroment },
            ],
        };
    }
}
FormBuilderModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    FBRendererComponent,
                ],
                imports: [
                    BsDatepickerModule.forRoot(),
                    PaginationModule.forRoot(),
                    TranslateModule.forRoot(),
                    AccordionModule.forRoot(),
                    TooltipModule.forRoot(),
                    ModalModule.forRoot(),
                    TabsModule.forRoot(),
                    NgxPaginationModule,
                    BrowserModule,
                    HttpClientModule,
                    FormsModule,
                    ReactiveFormsModule,
                ],
                providers: [
                    ApiService,
                    StorageService,
                    LanguageService,
                    FVNotificationService,
                    LowerTranslatePipe,
                ],
                schemas: [CUSTOM_ELEMENTS_SCHEMA],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Debugger {
    constructor() { }
    /**
     * @param {?} enviroment
     * @return {?}
     */
    init(enviroment) {
        this.enviroment = enviroment;
        this.showDebug = enviroment.debug;
    }
    /**
     * @param {...?} messages
     * @return {?}
     */
    log(...messages) {
        this.action('log', ...messages);
    }
    /**
     * @param {...?} messages
     * @return {?}
     */
    warn(...messages) {
        this.action('warn', ...messages);
    }
    /**
     * @param {...?} messages
     * @return {?}
     */
    error(...messages) {
        this.action('error', ...messages);
    }
    /**
     * @param {...?} messages
     * @return {?}
     */
    info(...messages) {
        this.action('info', ...messages);
    }
    /**
     * @param {...?} messages
     * @return {?}
     */
    debug(...messages) {
        this.action('debug', ...messages);
    }
    /**
     * @param {?} method
     * @param {...?} messages
     * @return {?}
     */
    action(method, ...messages) {
        if (this.showDebug) {
            console[method](...messages);
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { ApiResponse, ApiService, LanguageService as FVLanguageService, FVNotificationService, FormBuilderModule, Breadcrumbps, ComponentConfig, Control, ControlType, Dock, DockItem, Extract, Field, Item, Model, Path, Proxy, Tab, Task, FVDoneResponse, FVEnviroment, GridComponentConfig, Store, GridColumn, EditHandlers, GridConfig, Lang, Menu, Folder, Debugger, FBRendererComponent as ɵe, FVEnviroment as ɵd, FVEnviroment as ɵb, Lang as ɵc, LowerTranslatePipe as ɵf, StorageService as ɵa };
//# sourceMappingURL=ngx-formbuilder.js.map
